"use strict";
/***********************************************************************
 * YoLink temperature / humidity sensor device support
 *
 * Copyright (c) 2022 David Kerr
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.mqttThermoHydroDevice = exports.initThermoHydroDevice = void 0;
Error.stackTraceLimit = 100;
/***********************************************************************
 * initThermoHydroDevice
 * Initialise the temperature and humidity device services.
 */
async function initThermoHydroDevice() {
    const platform = this.platform;
    const accessory = this.accessory;
    const device = accessory.context.device;
    if (this.config.hide === 'thermo') {
        platform.log.info(`Hide Thermometer service because config.[${device.deviceId}].hide is set to "thermo"`);
    }
    else {
        // Not trying to hide the thermometer service.
        this.thermoService = accessory.getService(platform.Service.TemperatureSensor)
            || accessory.addService(platform.Service.TemperatureSensor);
        this.thermoService.setCharacteristic(platform.Characteristic.Name, device.name);
        this.thermoService.getCharacteristic(platform.Characteristic.CurrentTemperature)
            .onGet(handleThermoGet.bind(this));
    }
    if (this.config.hide === 'hydro') {
        platform.log.info(`Hide Hydrometer service because config.[${device.deviceId}].hide is set to "hydro"`);
    }
    else {
        // Not trying to hide the hydrometer service.
        this.hydroService = accessory.getService(platform.Service.HumiditySensor)
            || accessory.addService(platform.Service.HumiditySensor);
        this.hydroService.setCharacteristic(platform.Characteristic.Name, device.name);
        this.hydroService.getCharacteristic(platform.Characteristic.CurrentRelativeHumidity)
            .onGet(handleHydroGet.bind(this));
    }
    // Call get handler to initialize data fields to current state and set
    // timer to regularly update the data.  We only need to call one of these
    // as each fetches both data.
    if (this.config.hide === 'thermo') {
        this.refreshDataTimer(handleHydroGet.bind(this));
    }
    else {
        this.refreshDataTimer(handleThermoGet.bind(this));
    }
}
exports.initThermoHydroDevice = initThermoHydroDevice;
/***********************************************************************
 * handleGet
 *
 * This is an example of JSON object returned.
 * The only alarm state we can act on is the low battery, HomeKit does not
 * have alarm/alert state for high/low temperature or humidity, that would
 * be handled internally within HomeKit based on data returned.
 *
 *  {
 *    "online": true,
 *    "state": {
 *      "alarm": {
 *        "lowBattery": false,
 *        "lowTemp": false,
 *        "highTemp": false,
 *        "lowHumidity": false,
 *        "highHumidity": false,
 *        "period": false,
 *        "code": 0
 *      },
 *      "battery": 4,
 *      "batteryType": "Li",
 *      "humidity": 47.2,
 *      "humidityCorrection": 0,
 *      "humidityLimit": {
 *        "max": 50,
 *        "min": 0
 *      },
 *      "interval": 10,
 *      "mode": "f",
 *      "state": "normal",
 *      "tempCorrection": 0,
 *      "tempLimit": {
 *        "max": 35,
 *        "min": 18
 *      },
 *      "temperature": 25.4,
 *      "version": "0508"
 *    },
 *    "deviceId": "abcdef1234567890",
 *    "reportAt": "2022-07-22T15:32:15.899Z"
 *  }
 */
async function handleGet() {
    const platform = this.platform;
    // serialize access to device data.
    const releaseSemaphore = await this.deviceSemaphore.acquire();
    let rc = 0;
    try {
        const device = this.accessory.context.device;
        if (await this.checkDeviceState(platform, device) && device.data.online) {
            if (this.thermoService) {
                this.thermoService
                    .updateCharacteristic(platform.Characteristic.StatusLowBattery, ((device.data.state.battery <= 1) || (device.data.state.alarm.lowBattery))
                    ? platform.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_LOW
                    : platform.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_NORMAL)
                    .updateCharacteristic(platform.Characteristic.StatusActive, true)
                    .updateCharacteristic(platform.Characteristic.StatusFault, false);
            }
            if (this.hydroService) {
                this.hydroService
                    .updateCharacteristic(platform.Characteristic.StatusLowBattery, ((device.data.state.battery <= 1) || (device.data.state.alarm.lowBattery))
                    ? platform.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_LOW
                    : platform.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_NORMAL)
                    .updateCharacteristic(platform.Characteristic.StatusActive, true)
                    .updateCharacteristic(platform.Characteristic.StatusFault, false);
            }
            if ((device.data.state.battery <= 1) || (device.data.state.alarm.lowBattery)) {
                platform.log.warn(`Device ${this.deviceMsgName} reports battery < 25%`);
            }
            rc = device.data.state.temperature;
        }
        else {
            platform.log.error(`Device offline or other error for ${this.deviceMsgName}`);
            if (this.thermoService) {
                this.thermoService
                    .updateCharacteristic(platform.Characteristic.StatusActive, false)
                    .updateCharacteristic(platform.Characteristic.StatusFault, true);
            }
            if (this.hydroService) {
                this.hydroService
                    .updateCharacteristic(platform.Characteristic.StatusActive, false)
                    .updateCharacteristic(platform.Characteristic.StatusFault, true);
            }
        }
    }
    catch (e) {
        const msg = (e instanceof Error) ? e.stack : e;
        platform.log.error('Error in ThermoHydroDevice handleGet' + platform.reportError + msg);
    }
    finally {
        await releaseSemaphore();
    }
    return (rc);
}
/***********************************************************************
 * handleThermoGet
 * Call handleGet and return the temperature.
 */
async function handleThermoGet() {
    await handleGet.bind(this)();
    this.platform.liteLog(`Temperature for ${this.deviceMsgName} is: ${this.accessory.context.device.data.state.temperature}`);
    return (this.accessory.context.device.data.state.temperature);
}
/***********************************************************************
 * handleHydroGet
 * Call handleGet and return the humidity.
 */
async function handleHydroGet() {
    await handleGet.bind(this)();
    this.platform.liteLog(`Humidity for ${this.deviceMsgName} is: ${this.accessory.context.device.data.state.humidity}`);
    return (this.accessory.context.device.data.state.humidity);
}
/***********************************************************************
 * mqttThermoHydroDevice
 * Handle message received from MQTT server.
 *
 * This is an example of JSON object returned.
 * The only alarm state we can act on is the low battery, HomeKit does not
 * have alarm/alert state for high/low temperature or humidity, that would
 * be handled internally within HomeKit based on data returned.
 *  {
 *    "event": "THSensor.Report",
 *    "time": 1658507535922,
 *    "msgid": "1658507535922",
 *    "data": {
 *      "state": "normal",
 *      "alarm": {
 *        "lowBattery": false,
 *        "lowTemp": false,
 *        "highTemp": false,
 *        "lowHumidity": false,
 *        "highHumidity": false,
 *        "period": false,
 *        "code": 0
 *      },
 *      "battery": 4,
 *      "mode": "f",
 *      "interval": 10,
 *      "temperature": 25.5,
 *      "humidity": 48.1,
 *      "tempLimit": {
 *        "max": 35,
 *        "min": 18
 *      },
 *      "humidityLimit": {
 *        "max": 50,
 *        "min": 0
 *      },
 *      "tempCorrection": 0,
 *      "humidityCorrection": 0,
 *      "version": "0508",
 *      "batteryType": "Li",
 *      "loraInfo": {
 *        "signal": -64,
 *        "gatewayId": "abcdef1234567890",
 *        "gateways": 1
 *      }
 *    },
 *    "deviceId": "abcdef1234567890"
 *  }
 */
async function mqttThermoHydroDevice(message) {
    const platform = this.platform;
    // serialize access to device data.
    const releaseSemaphore = await this.deviceSemaphore.acquire();
    try {
        const device = this.accessory.context.device;
        device.updateTime = Math.floor(new Date().getTime() / 1000) + this.config.refreshAfter;
        const event = message.event.split('.');
        switch (event[1]) {
            case 'Alert':
            // I can see no way in HomeKit documentation for a thermo/hyrdro sensor
            // to generate an alert.  I think bounds testing / alerting all has to be
            // handled within HomeKit.
            // falls through
            case 'Report':
                if (!device.data) {
                    // in rare conditions (error conditions returned from YoLink) data object will be undefined or null.
                    platform.log.warn(`Device ${this.deviceMsgName} has no data field, is device offline?`);
                    break;
                }
                // if we received a message then device must be online
                device.data.online = true;
                // Merge received data into existing data object
                Object.assign(device.data.state, message.data);
                if (this.thermoService) {
                    this.thermoService.updateCharacteristic(platform.Characteristic.StatusLowBattery, ((device.data.state.battery <= 1) || (device.data.state.alarm.lowBattery))
                        ? platform.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_LOW
                        : platform.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_NORMAL);
                    this.thermoService.updateCharacteristic(platform.Characteristic.CurrentTemperature, message.data.temperature);
                }
                if (this.hydroService) {
                    this.hydroService.updateCharacteristic(platform.Characteristic.StatusLowBattery, ((device.data.state.battery <= 1) || (device.data.state.alarm.lowBattery))
                        ? platform.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_LOW
                        : platform.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_NORMAL);
                    this.hydroService.updateCharacteristic(platform.Characteristic.CurrentRelativeHumidity, message.data.humidity);
                }
                if ((device.data.state.battery <= 1) || (device.data.state.alarm.lowBattery)) {
                    platform.log.warn(`Device ${this.deviceMsgName} reports battery < 25%`);
                }
                break;
            default:
                platform.log.warn('Unsupported mqtt event: \'' + message.event + '\'' + platform.reportError + JSON.stringify(message));
        }
    }
    catch (e) {
        const msg = (e instanceof Error) ? e.stack : e;
        platform.log.error('Error in mqttThermoHydroDevice' + platform.reportError + msg);
    }
    finally {
        await releaseSemaphore();
    }
}
exports.mqttThermoHydroDevice = mqttThermoHydroDevice;
//# sourceMappingURL=thermoHydroDevice.js.map